# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0.  If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
# Copyright 1997 - July 2008 CWI, August 2008 - 2016 MonetDB B.V.

import ctypes
import os
import sys

from monetdblite import monetize
from monetdblite import exceptions

PY3 = sys.version_info[0] >= 3

basedir = os.path.dirname(os.path.abspath(__file__))
libs = [x for x in os.listdir(basedir) if
        x.startswith('libmonetdb5') and
        (x.endswith('.so') or x.endswith('.dylib') or x.endswith('.dll'))]

if len(libs) == 0:
    raise Exception('Could not locate library file "libmonetdb5.[dll|so|dylib] in folder %s' % basedir)

try:
    import numpy
except ImportError:
    raise Exception('MonetDBLite requires numpy but import of numpy failed')

if os.name == 'nt':
    os.environ["PATH"] += os.pathsep + os.path.dirname(os.path.abspath(__file__))

dll = ctypes.PyDLL(os.path.join(os.path.dirname(os.path.abspath(__file__)),
                                libs[0]), mode=ctypes.RTLD_GLOBAL)
dll.python_monetdblite_init.argtypes = []
dll.python_monetdblite_init.restype = None
dll.python_monetdblite_init()

dll.python_monetdb_init.argtypes = [ctypes.c_char_p, ctypes.c_int]
dll.python_monetdb_init.restype = ctypes.py_object

dll.python_monetdb_sql.argtypes = [ctypes.c_void_p, ctypes.c_char_p]
dll.python_monetdb_sql.restype = ctypes.py_object

dll.python_monetdb_insert.argtypes = [
    ctypes.c_void_p,
    ctypes.c_char_p,
    ctypes.c_char_p,
    ctypes.py_object
]
dll.python_monetdb_insert.restype = ctypes.py_object

dll.python_monetdb_set_autocommit.argtypes = [ctypes.c_void_p, ctypes.c_int]
dll.python_monetdb_set_autocommit.restype = ctypes.py_object

dll.python_monetdb_client.argtypes = []
dll.python_monetdb_client.restype = ctypes.c_void_p

dll.python_monetdb_disconnect.argtypes = [ctypes.c_void_p]
dll.python_monetdb_disconnect.restype = None

dll.python_monetdb_shutdown.argtypes = []
dll.python_monetdb_shutdown.restype = None

# Global variables. Every function *assinging* to them needs to declare
# them. See: https://docs.python.org/3/reference/simple_stmts.html#the-global-statement
MONETDBLITE_IS_INITIALIZED = False
MONETDBLITE_CURRENT_DATABASE = None


class PyClient:
    def __init__(self):
        self.client = dll.python_monetdb_client()

    def get_client(self):
        return self.client


if PY3:
    def utf8_encode(instr):
        if instr is None:
            return None
        if isinstance(instr, str):
            return instr.encode('utf-8')
        return instr
else:
    def utf8_encode(instr):
        return instr


def __throw_exception(str):
    raise exceptions.DatabaseError(str.replace('MALException:', ''))


def init(directory):
    global MONETDBLITE_CURRENT_DATABASE
    global MONETDBLITE_IS_INITIALIZED
    if directory == ':memory:':
        directory = None
    else:
        directory = utf8_encode(directory)
    """Initializes the MonetDBLite database in the specified directory."""
    retval = dll.python_monetdb_init(directory, 0)
    if retval is not None:
        raise __throw_exception(str(retval) + ' in ' + str(directory))
    MONETDBLITE_IS_INITIALIZED = True
    MONETDBLITE_CURRENT_DATABASE = directory


def is_initialized():
    return MONETDBLITE_IS_INITIALIZED


def dbpath():
    return MONETDBLITE_CURRENT_DATABASE


def sql(query, client=None):
    """Executes a SQL statement on the database if the database
       has been initialized. If no client context is provided,
       the default client context is used. Otherwise the specified
       client context is used to execute the query."""

    if client is not None and not isinstance(client, PyClient):
        raise __throw_exception("client must be of type PyClient")
    client_object = None
    if client is not None:
        client_object = client.get_client()

    retval = dll.python_monetdb_sql(client_object, utf8_encode(query))
    if isinstance(retval, str):
        raise __throw_exception(str(retval))
    else:
        return retval


def __convert_pandas_to_numpy_dict__(df):
    import pandas, numpy
    if type(df) == pandas.DataFrame:
        res = {}
        for tpl in df.to_dict().items():
            res[tpl[0]] = numpy.array(tpl[1].values())
        return res
    return df


def insert(table, values, schema=None, client=None):
    """Inserts a set of values into the specified table. The values must
       be either a pandas dataframe or a dictionary of values. If no schema
       is specified, the "sys" schema is used. If no client context is
       provided, the default client context is used. """

    if client is not None and not isinstance(client, PyClient):
        raise __throw_exception("client must be of type PyClient")
    client_object = None
    if client is not None:
        client_object = client.get_client()

    if not isinstance(values, dict):
        values = __convert_pandas_to_numpy_dict__(values)
    else:
        vals = {}
        for tpl in values.items():
            vals[tpl[0]] = numpy.array(tpl[1])
        values = vals
    retval = dll.python_monetdb_insert(client_object, utf8_encode(schema),
                                       utf8_encode(table), values)
    if isinstance(retval, str):
        raise __throw_exception(str(retval))
    else:
        return retval


def create(table, values, schema=None, client=None):
    """Creates a table from a set of values or a pandas DataFrame."""
    column_types = []

    if not isinstance(values, dict):
        values = __convert_pandas_to_numpy_dict__(values)
    else:
        vals = {}
        for tpl in values.items():
            vals[tpl[0]] = numpy.array(tpl[1])
        values = vals
    if schema is None:
        schema = "sys"
    for key, value in values.items():
        arr = numpy.array(value)
        if arr.dtype == numpy.bool:
            column_type = "BOOLEAN"
        elif arr.dtype == numpy.int8:
            column_type = 'TINYINT'
        elif arr.dtype == numpy.int16 or arr.dtype == numpy.uint8:
            column_type = 'SMALLINT'
        elif arr.dtype == numpy.int32 or arr.dtype == numpy.uint16:
            column_type = 'INT'
        elif arr.dtype == numpy.int64 or arr.dtype == numpy.uint32 or arr.dtype == numpy.uint64:
            column_type = 'BIGINT'
        elif arr.dtype == numpy.float32:
            column_type = 'REAL'
        elif arr.dtype == numpy.float64:
            column_type = 'DOUBLE'
        elif numpy.issubdtype(arr.dtype, numpy.str_) or numpy.issubdtype(arr.dtype, numpy.unicode_):
            column_type = 'STRING'
        else:
            raise Exception('Unsupported dtype: %s' % (str(arr.dtype)))
        column_types.append(column_type)
    query = 'CREATE TABLE %s.%s (' % (monetize.monet_identifier_escape(schema), monetize.monet_identifier_escape(table))
    index = 0
    for key in values.keys():
        query += '%s %s, ' % (monetize.monet_identifier_escape(key), column_types[index])
        index += 1
    query = query[:-2] + ");"
    # create the table
    sql(query, client=client)
    # insert the data into the table
    insert(table, values, schema=schema, client=client)


def connect():
    return PyClient()


def disconnect(client):
    if not isinstance(client, PyClient):
        raise __throw_exception("client must be of type PyClient")
    dll.python_monetdb_disconnect(client.get_client())


def shutdown():
    global MONETDBLITE_CURRENT_DATABASE
    global MONETDBLITE_IS_INITIALIZED

    MONETDBLITE_IS_INITIALIZED = False
    MONETDBLITE_CURRENT_DATABASE = None
    retval = dll.python_monetdb_shutdown()
    if isinstance(retval, str):
        raise __throw_exception(str(retval))
    else:
        return retval


def set_autocommit(val, client=None):
    if client is not None and not isinstance(client, PyClient):
        raise __throw_exception("client must be of type PyClient")
    client_object = None
    if client is not None:
        client_object = client.get_client()

    retval = dll.python_monetdb_set_autocommit(client_object, val)
    if isinstance(retval, str):
        raise __throw_exception(str(retval))
    else:
        return retval



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0.  If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
# Copyright 1997 - July 2008 CWI, August 2008 - 2016 MonetDB B.V.
from itertools import repeat

from monetdblite.exceptions import *
from monetdblite import embeddedmonetdb
from monetdblite import monetize
import numpy


class Cursor(object):
    """This object represents a database cursor, which is used to manage
    the context of a fetch operation. Cursors created from the same
    connection are not isolated, i.e., any changes done to the
    database by a cursor are immediately visible by the other
    cursors"""

    def __init__(self, connection):
        """This read-only attribute return a reference to the Connection
        object on which the cursor was created."""
        self.connection = connection

        # last executed operation (query)
        self.operation = ""

        # This read/write attribute specifies the number of rows to
        # fetch at a time with .fetchmany()
        self.arraysize = connection.replysize

        # This read-only attribute specifies the number of rows that
        # the last .execute*() produced (for DQL statements like
        # 'select') or affected (for DML statements like 'update' or
        # 'insert').
        #
        # The attribute is -1 in case no .execute*() has been
        # performed on the cursor or the rowcount of the last
        # operation is cannot be determined by the interface.
        self.rowcount = -1

        # This read-only attribute is a sequence of 7-item
        # sequences.
        #
        # Each of these sequences contains information describing
        # one result column:
        #
        #   (name,
        #    type_code,
        #    display_size,
        #    internal_size,
        #    precision,
        #    scale,
        #    null_ok)
        #
        # This attribute will be None for operations that
        # do not return rows or if the cursor has not had an
        # operation invoked via the .execute*() method yet.
        self.description = None

        # This read-only attribute indicates at which row
        # we currently are
        self.rownumber = -1

        self.__executed = None

        # the offset of the current resultset in the total resultset
        self.__offset = 0

        # the resultsets
        self.__results = []

        # This is a Python list object to which the interface appends
        # tuples (exception class, exception value) for all messages
        # which the interfaces receives from the underlying database for
        # this cursor.
        #
        # The list is cleared by all standard cursor methods calls (prior
        # to executing the call) except for the .fetch*() calls
        # automatically to avoid excessive memory usage and can also be
        # cleared by executing "del cursor.messages[:]".
        #
        # All error and warning messages generated by the database are
        # placed into this list, so checking the list allows the user to
        # verify correct operation of the method calls.
        self.messages = []

        # This read-only attribute provides the rowid of the last
        # modified row (most databases return a rowid only when a single
        # INSERT operation is performed). If the operation does not set
        # a rowid or if the database does not support rowids, this
        # attribute should be set to None.
        #
        # The semantics of .lastrowid are undefined in case the last
        # executed statement modified more than one row, e.g. when
        # using INSERT with .executemany().
        self.lastrowid = None

    def __check_executed(self):
        if not self.__executed:
            self.__exception_handler(ProgrammingError, "do a execute() first")


    def close(self):
        """ Close the cursor now (rather than whenever __del__ is
        called).  The cursor will be unusable from this point
        forward; an Error (or subclass) exception will be raised
        if any operation is attempted with the cursor."""
        if self.connection != None:
            self.connection.remove_cursor(self)
            self.connection = None

    def execute(self, operation, parameters=None, discard_previous=True):
        """Prepare and execute a database operation (query or
        command).  Parameters may be provided as mapping and
        will be bound to variables in the operation.
        """
        if not self.connection:
            self.__exception_handler(ProgrammingError, "cursor is closed")

        # clear message history
        self.messages = []

        # convert to utf-8
        operation = operation

        self.operation = operation

        query = ""
        if parameters:
            if isinstance(parameters, dict):
                query = operation % dict([(k, monetize.convert(v))
                                         for (k, v) in parameters.items()])
            elif type(parameters) == list or type(parameters) == tuple:
                query = operation % tuple([monetize.convert(item) for item
                                           in parameters])
            elif isinstance(parameters, str):
                query = operation % monetize.convert(parameters)
            else:
                msg = "Parameters should be None, dict or list, now it is %s"
                self.__exception_handler(ValueError, msg % type(parameters))
        else:
            query = operation

        if discard_previous:
            self.__results = []

        query = embeddedmonetdb.utf8_encode(query)
        result = self.connection.execute(query)
        result_set_length = 0 if type(result) != type({}) or len(result) == 0 else len(result[list(result.keys())[0]])

        if result:
            keys, dtypes = zip(*((k, v.dtype) for k, v in result.items()))
            # description fields: name, type_code, display_size, internal_size, precision, scale, null_ok
            self.description = list(zip(keys, dtypes, repeat(None), repeat(None), repeat(None), repeat(None), repeat(None)))
            self.__results.append([keys, result, result_set_length])
        else:
            self.description = None

        self.rowcount = result_set_length
        self.rownumber = 0
        self.__executed = operation
        return self.rowcount

    def executemany(self, operation, seq_of_parameters):
        """Prepare a database operation (query or command) and then
        execute it against all parameter sequences or mappings
        found in the sequence seq_of_parameters.
        It will return the number or rows affected
        """

        count = 0
        for parameters in seq_of_parameters:
            count += self.execute(operation, parameters, False)
        self.rowcount = count
        return count

    def fetchone(self):
        """Fetch the next row of a query result set, returning a
        single sequence, or None when no more data is available."""

        self.__check_executed()

        if self.rownumber >= self.rowcount:
            return None

        if self.rownumber >= (self.__offset + self.__results[0][2]):
            self.nextset()

        rownumber = self.rownumber - self.__offset;
        result = [self.__results[0][1][key][rownumber] for key in self.__results[0][0]]
        result = [x if x is not numpy.ma.masked else None for x in result]
        self.rownumber += 1
        return result

    def fetchmany(self, size=None):
        """Fetch the next set of rows of a query result, returning a
        sequence of sequences (e.g. a list of tuples). An empty
        sequence is returned when no more rows are available.
        The number of rows to fetch per call is specified by the
        parameter.  If it is not given, the cursor's arraysize
        determines the number of rows to be fetched. The method
        should try to fetch as many rows as indicated by the size
        parameter. If this is not possible due to the specified
        number of rows not being available, fewer rows may be
        returned.
        An Error (or subclass) exception is raised if the previous
        call to .execute*() did not produce any result set or no
        call was issued yet.
        Note there are performance considerations involved with
        the size parameter.  For optimal performance, it is
        usually best to use the arraysize attribute.  If the size
        parameter is used, then it is best for it to retain the
        same value from one .fetchmany() call to the next."""

        self.__check_executed()

        if size is None:
            size = self.arraysize

        if self.rownumber >= self.rowcount:
            return []

        result = []
        for i in range(size):
            row = self.fetchone()
            if row is None:
                break
            result.append(row)
        return result

    def fetchall(self):
        """Fetch all (remaining) rows of a query result, returning
        them as a sequence of sequences (e.g. a list of tuples).
        Note that the cursor's arraysize attribute can affect the
        performance of this operation.
        An Error (or subclass) exception is raised if the previous
        call to .execute*() did not produce any result set or no
        call was issued yet."""

        self.__check_executed()

        result = []
        while True:
            row = self.fetchone()
            if row is None:
                break
            result.append(row)
        return result

    def fetchnumpy(self):
        """Fetches the current result set as a dictionary of NumPy 
        arrays. This is the most efficient fetch function as this
        is the way the connector actually stores the result, so
        no conversion to Python objects is necessary.
        """
        self.__check_executed()

        if len(self.__results) == 0:
            return None

        result = self.__results[0][1]
        del self.__results[0]
        return result

    def fetchdf(self):
        """Fetches the current result set as a Pandas DataFrame.
        """
        import pandas
        result = self.fetchnumpy()
        if result is None:
            return None
        return pandas.DataFrame.from_dict(result)

    def insert(self, table, values, schema=None):
        """Inserts a set of values into the specified table. The values must
           be either a pandas dataframe or a dictionary of values. If no schema
           is specified, the "sys" schema is used. If no client context is 
           provided, the default client context is used. """
        if not self.connection:
            self.__exception_handler(ProgrammingError, "cursor is closed")
        return embeddedmonetdb.insert(table, values, schema=schema, client=self.connection.get_connection())

    def create(self, table, values, schema=None):
        """Creates a table from a set of values or a pandas DataFrame."""
        if not self.connection:
            self.__exception_handler(ProgrammingError, "cursor is closed")
        return embeddedmonetdb.create(table, values, schema=schema, client=self.connection.get_connection())

    def commit(self):
        """Commits the current transaction."""
        if not self.connection:
            self.__exception_handler(ProgrammingError, "cursor is closed")
        self.connection.commit()

    def rollback(self):
        """Rollbacks the current transaction."""
        if not self.connection:
            self.__exception_handler(ProgrammingError, "cursor is closed")
        self.connection.rollback()

    def transaction(self):
        """Starts a transaction. Non-Standard."""
        if not self.connection:
            self.__exception_handler(ProgrammingError, "cursor is closed")
        self.connection.transaction()

    def nextset(self):
        """This method will make the cursor skip to the next
        available set, discarding any remaining rows from the
        current set.
        If there are no more sets, the method returns
        None. Otherwise, it returns a true value and subsequent
        calls to the fetch methods will return rows from the next
        result set.
        An Error (or subclass) exception is raised if the previous
        call to .execute*() did not produce any result set or no
        call was issued yet."""

        self.__check_executed()

        if self.rownumber >= self.rowcount:
            return False

        self.__offset += self.__results[0][2]
        del self.__results[0]
        return True

    def setinputsizes(self, sizes):
        """
        This method would be used before the .execute*() method
        is invoked to reserve memory. This implementation doesn't
        use this.
        """
        pass

    def setoutputsize(self, size, column=None):
        """
        Set a column buffer size for fetches of large columns
        This implementation doesn't use this
        """
        pass

    def __iter__(self):
        return self

    def next(self):
        row = self.fetchone()
        if not row:
            raise StopIteration
        return row

    def scroll(self, value, mode='relative'):
        """Scroll the cursor in the result set to a new position according
        to mode.
        If mode is 'relative' (default), value is taken as offset to
        the current position in the result set, if set to 'absolute',
        value states an absolute target position.
        An IndexError is raised in case a scroll operation would
        leave the result set.
        """
        self.__check_executed()

        if mode not in ['relative', 'absolute']:
            msg = "unknown mode '%s'" % mode
            self.__exception_handler(ProgrammingError, msg)

        if mode == 'relative':
            value += self.rownumber

        if value > self.rowcount:
            self.__exception_handler(IndexError,
                                     "value beyond length of resultset")
        self.__offset = value

    def __exception_handler(self, exception_class, message):
        """ raises the exception specified by exception, and add the error
        to the message list """
        self.messages.append((exception_class, message))
        raise exception_class(message)




import os
import sys
import re
import zlib

# these includes are skipped
blacklist = ('remote', 'srvpool', 'mal_mapi', 'json', 'json_util', 'mcurl', 'xml', 'batxml', 'recycle', 'txtsim', 'tokenizer', 'zorder', '70_vault', '80_lsst', '80_udf', '23_skyserver', '24_zorder', '40_json', '80_udf')

def mal_include(filename):
    if os.path.isdir(filename):
        files = os.listdir(filename)
        files.sort()
        ret = ""
        for f in files:
            if f.endswith(".mal") and not f.startswith(blacklist):
                ret += mal_include(os.path.join(filename, f))
        return ret
    elif os.path.isfile(filename):
        print filename
        content = open(filename).read() + "\n"
        content = re.sub("^#.*$", "", content, flags=re.MULTILINE)
        while True:
            match = re.search("include (\\w+);", content)
            if (match == None): break
            modname = match.groups(0)[0]
            if not modname.startswith(blacklist):
                incfile = mal_include(modname + ".mal" if os.path.isfile(modname + ".mal") else modname)
            content = content[:match.start()] + incfile + content[match.end():]
        return content
    else:
        return ""

def to_hex(s, n=1024):
    result = "{"
    for chunk in [s[i:i+n] for i in range(0, len(s), n)]:
        result += ",".join(str(ord(c)) for c in chunk) +  ",\n"
    return "\n" + result + "0}"

os.chdir(sys.argv[1])
mi = mal_include("mal_init.mal")
#print(mi)
s = zlib.compress(mi, 9)
outf = open(sys.argv[2], "w")
outf.write("unsigned char mal_init_inline_arr[] = " + to_hex(s) + ";\n")
outf.write("unsigned char* mal_init_inline = 0;\n")

s = ""
files = os.listdir("createdb")
files.sort()
for f in files:
    if f.endswith(".sql") and not f.startswith(blacklist):
        print(f)
        s += open(os.path.join("createdb", f)).read() + "\n"
s = zlib.compress(s, 9)
outf.write("\nunsigned char createdb_inline_arr[] = " + to_hex(s) + ";\n")
outf.write("unsigned char* createdb_inline = 0;\n")




"""
MonetDBLite is an embedded database.

To set up a connection use monetdblite.connect()
"""
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0.  If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
#
# Copyright 1997 - July 2008 CWI, August 2008 - 2016 MonetDB B.V.

import pkg_resources
from monetdblite import connections
from monetdblite import cursors
from monetdblite import exceptions
from monetdblite import embeddedmonetdb

from monetdblite.connections import Connection
from monetdblite.exceptions import *


try:
    __version__ = pkg_resources.require("monetdblite")[0].version
except pkg_resources.DistributionNotFound:
    __version__ = "1.0rc"


apilevel = "2.0"
threadsafety = 0
paramstyle = "pyformat"


def connect(*args, **kwargs):
    return Connection(*args, **kwargs)

connect.__doc__ = Connection.__init__.__doc__


def init(*args, **kwargs):
    return embeddedmonetdb.init(*args, **kwargs)

init.__doc__ = embeddedmonetdb.init.__doc__

def sql(*args, **kwargs):
    return embeddedmonetdb.sql(*args, **kwargs)

sql.__doc__ = embeddedmonetdb.sql.__doc__


def insert(*args, **kwargs):
    return embeddedmonetdb.insert(*args, **kwargs)

insert.__doc__ = embeddedmonetdb.insert.__doc__

def create(*args, **kwargs):
    return embeddedmonetdb.create(*args, **kwargs)

create.__doc__ = embeddedmonetdb.create.__doc__

def connectclient(*args, **kwargs):
    return embeddedmonetdb.connect(*args, **kwargs)

connectclient.__doc__ = embeddedmonetdb.connect.__doc__

def disconnectclient(*args, **kwargs):
    return embeddedmonetdb.disconnect(*args, **kwargs)

disconnectclient.__doc__ = embeddedmonetdb.disconnect.__doc__

def shutdown(*args, **kwargs):
    return embeddedmonetdb.shutdown(*args, **kwargs)

shutdown.__doc__ = embeddedmonetdb.shutdown.__doc__



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from distutils import sysconfig
import ntpath
import os
import sys
from shutil import copyfile
import subprocess
import platform
import re
from setuptools import setup, find_packages

PY3 = sys.version_info[0] >= 3
bits = platform.architecture()[0]

pypi_upload = 'MONETDBLITE_PIP_UPLOAD' in os.environ


def get_python_include_flags():
    pyver = sysconfig.get_config_var('VERSION')
    getvar = sysconfig.get_config_var
    flags = ['-I' + sysconfig.get_python_inc(),
             '-I' + sysconfig.get_python_inc(plat_specific=True)]
    flags.extend(getvar('CFLAGS').split())
    flags.append('-I' + numpy.get_include())
    return ' '.join(flags)


def get_python_link_flags():
    pyver = sysconfig.get_config_var('VERSION')
    getvar = sysconfig.get_config_var
    libs = ['-L' + sysconfig.get_config_var('LIBDIR') +
           ' -l' + sysconfig.get_config_var('LIBRARY').replace('.a', '').replace('.so', '').replace('.dll', '').replace('.so', '').replace('lib', '')]
    libs += getvar('LIBS').split()
    libs += getvar('SYSLIBS').split()
    if not getvar('Py_ENABLE_SHARED'):
        libs.insert(0, '-L' + getvar('LIBPL'))
    if not getvar('PYTHONFRAMEWORK'):
        libs.extend(getvar('LINKFORSHARED').split())
    return re.sub('\S+stack_size\S+', '', ' '.join(libs))


basedir = os.path.dirname(os.path.realpath(__file__))
if os.name == 'nt':
    basedir = 'C:\\monetdblite\\build\\ffd2a0a35d2fed128a199f8537628c58a43c49ab\\python%d-%s' % (3 if PY3 else 2, platform.architecture()[0])
    print(basedir)
    monetdb_shared_lib_base = "libmonetdb5.dll"
    monetdb_shared_lib = os.path.join(basedir, monetdb_shared_lib_base)
    so_extension = '.dll'
else:
    try:
        import numpy
    except ImportError:
        print('Building MonetDBLite from source requires NumPy to be installed.')
        exit(1)

    # build the dynamic library (.so/.dylib) on linux/osx
    os.environ['MONETDBLITE_PYTHON_INCLUDE_FLAGS'] = get_python_include_flags()
    os.environ['MONETDBLITE_PYTHON_LINK_FLAGS'] = get_python_link_flags()
    current_directory = os.getcwd()
    os.chdir(basedir)
    if not pypi_upload:
        # don't build the package if we are uploading to pip
        proc = subprocess.Popen(['make -C src -j'], stderr=subprocess.PIPE, shell=True)
        if proc.wait() != 0:
            error = proc.stderr.read()
            raise Exception('Failed to compile MonetDBLite sources: ' +
                ("No error specified" if error is None else
                (error.decode('utf8') if PY3 else error)))
    so_extension = '.so'
    os.chdir(current_directory)
    monetdb_shared_lib_base = "libmonetdb5" + so_extension
    monetdb_shared_lib = os.path.join(basedir, 'src', 'build', monetdb_shared_lib_base)

final_shared_library = os.path.join('monetdblite', monetdb_shared_lib_base)

long_description = ""
if not pypi_upload:
    # don't include the [so|dylib|dll] in the packaged version uploaded to pip
    copyfile(monetdb_shared_lib, final_shared_library)

else:
    os.chdir(os.path.join(basedir, 'src'))
    os.system('make clean')
    os.system('rm ../monetdblite/*.so ../monetdblite/*.dylib ../monetdblite/*.dll')
    os.chdir(current_directory)
    try:
        import pypandoc
        long_description = pypandoc.convert('README.md', 'rst')
    except(IOError, ImportError):
        long_description = open('README.md').read()

# now actually create the package
# the package is a single C file that only dynamically
# loads functions from libmonetdb5.[so|dylib|dll]
setup(
    name = "monetdblite",
    version = '0.6.0',
    description = 'Embedded MonetDB Python Database.',
    author = 'Mark Raasveldt, Hannes Mühleisen',
    author_email = 'm.raasveldt@cwi.nl',
    keywords = 'MonetDB, MonetDBLite, Database',
    packages = ['monetdblite'],
    package_data={
        'monetdblite': ['*.so', '*.dylib', '*.dll'],
    },
    url="https://github.com/hannesmuehleisen/MonetDBLite-Python",
    long_description = long_description
    )
